authorization_urls = dict(
    homologation = 'https://wsaahomo.afip.gov.ar/ws/services/LoginCms?wsdl',
    production = 'https://wsaa.afip.gov.ar/ws/services/LoginCms?wsdl', 
)

service_urls = dict(
    wsfev1_homologation = 'https://wswhomo.afip.gov.ar/wsfev1/service.asmx?wsdl',
    wsfev1_production = 'https://servicios1.afip.gov.ar/wsfev1/service.asmx?wsdl', 
    wsfexv1_homologation = 'https://wswhomo.afip.gov.ar/wsfexv1/service.asmx?wsdl',
    wsfexv1_production = 'https://servicios1.afip.gov.ar/wsfexv1/service.asmx?wsdl',
    ws_sr_padron_a4_homologation = 'https://awshomo.afip.gov.ar/sr-padron/webservices/personaServiceA4?WSDL',
    ws_sr_padron_a4_production = 'https://aws.afip.gov.ar/sr-padron/webservices/personaServiceA4?WSDL',
)